@@ -1,5 +1,5 @@
 /*
- * Copyright 1997, 1998 Phil Schwan <pschwan@cmu.edu>
+ * Copyright 1997-1999 Phil Schwan <pschwan@cmu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,6 +99,10 @@ static void config_daily_sort_pref(pointers_t *, char *);
 static void config_size_sort_pref(pointers_t *, char *);
 static void config_monthly_sort_pref(pointers_t *, char *);
 static void config_strict_check(pointers_t *);
+static void config_begin_date(pointers_t *, char *);
+static void config_end_date(pointers_t *, char *);
+static void config_chunk_input(pointers_t *, char *);
+static void config_refresh(pointers_t *, char *);
 
 static config_function config_commands[] =
 {
@@ -150,6 +154,10 @@ static config_function config_commands[] =
   {"SIZE_SORT_PREF", (void *)config_size_sort_pref, 1},
   {"MONTHLY_SORT_PREF", (void *)config_monthly_sort_pref, 1},
   {"STRICT_CHECK", (void *)config_strict_check, 0},
+  {"BEGIN_DATE", (void *)config_begin_date, 1},
+  {"END_DATE", (void *)config_end_date, 1},
+  {"CHUNK_INPUT", (void *)config_chunk_input, 1},
+  {"REFRESH", (void *)config_refresh, 1},
   {"", NULL, 0}
 };
 
@@ -665,12 +673,80 @@ config_monthly_sort_pref(pointers_t * pointers, char * pref)
 
 
 static void
-config_strict_check(pointers_t *pointers)
+config_strict_check(pointers_t * pointers)
 {
   pointers->config->strict_check = 1;
 } /* config_strict_check */
 
 
+static void
+config_begin_date(pointers_t * pointers, char * bar)
+{
+  if (strlen(bar) != 20)
+    {
+      fprintf(stderr, "fatal: the BEGIN_DATE must be in the "
+	      "Jan DD HH:MM:ss YYYY format\n");
+      exit(1);
+    }
+
+  if (bar[3] != ' ' || bar[6] != ' ' || bar[15] != ' ' || bar[9] != ':' ||
+      bar[12] != ':' || !isdigit(bar[5]) || !isdigit(bar[7]) ||
+      !isdigit(bar[8]) || !isdigit(bar[10]) || !isdigit(bar[11]) ||
+      !isdigit(bar[13]) || !isdigit(bar[14]) || !isdigit(bar[16]) ||
+      !isdigit(bar[17]) || !isdigit(bar[18]) || !isdigit(bar[19]) ||
+      !(bar[4] == ' ' || bar[4] == '0' || bar[4] == '1' || bar[4] == '2' ||
+	bar[4] == '3'))
+    {
+      fprintf(stderr, "fatal: the BEGIN_DATE must be in the "
+	      "Jan DD HH:MM:ss YYYY format\n");
+      exit(1);
+    }
+  memset(pointers->config->begin_date, ' ', 4);
+  memcpy(pointers->config->begin_date + 4, bar, 21);
+} /* config_begin_date */
+
+
+static void
+config_end_date(pointers_t * pointers, char * bar)
+{
+  if (strlen(bar) != 20)
+    {
+      fprintf(stderr, "fatal: the END_DATE must be in the "
+	      "Jan DD HH:MM:ss YYYY format\n");
+      exit(1);
+    }
+
+  if (bar[3] != ' ' || bar[6] != ' ' || bar[15] != ' ' || bar[9] != ':' ||
+      bar[12] != ':' || !isdigit(bar[5]) || !isdigit(bar[7]) ||
+      !isdigit(bar[8]) || !isdigit(bar[10]) || !isdigit(bar[11]) ||
+      !isdigit(bar[13]) || !isdigit(bar[14]) || !isdigit(bar[16]) ||
+      !isdigit(bar[17]) || !isdigit(bar[18]) || !isdigit(bar[19]) ||
+      !(bar[4] == ' ' || bar[4] == '0' || bar[4] == '1' || bar[4] == '2' ||
+	bar[4] == '3'))
+    {
+      fprintf(stderr, "fatal: the END_DATE must be in the "
+	      "Jan DD HH:MM:ss YYYY format\n");
+      exit(1);
+    }
+  memset(pointers->config->end_date, ' ', 4);
+  memcpy(pointers->config->end_date + 4, bar, 21);
+} /* config_end_date */
+
+
+static void
+config_chunk_input(pointers_t * pointers, char * lines)
+{
+  pointers->config->chunk_input = atoi(lines);
+} /* config_chunk_input */
+
+
+static void
+config_refresh(pointers_t * pointers, char * refresh)
+{
+  pointers->config->refresh = atoi(refresh);
+} /* config_refresh */
+
+
 /* parse_config_line returns 0 if there is no valid data, 1 if there is only a
  * valid "command", 2 if there is a valid "command" and a valid "arg1", 3 if
  * there is also a valid "arg2", and 4 if the command and all 3 args are
