@@ -1,5 +1,5 @@
 /*
- * Copyright 1997, 1998 Phil Schwan <pschwan@cmu.edu>
+ * Copyright 1997-1999 Phil Schwan <pschwan@cmu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -165,9 +165,13 @@ display_daily_totals(pointers_t * pointers)
       /* first for HTML */
       if (!pointers->config->no_html_headers && !pointers->config->single_page)
 	fprintf(html, "<HTML>"); 
-      if (!pointers->config->no_html_headers)
+      if (!pointers->config->no_html_headers) {
 	fprintf(html, "<TITLE>xferstats: %s to %s</TITLE>\n",
 		pointers->low_date, pointers->high_date);
+        if (pointers->config->refresh)
+	  fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+		  pointers->config->refresh);
+      }
       if (!pointers->config->no_html_headers && !pointers->config->single_page)
 	fprintf(html, "<BODY>");
 
@@ -237,10 +241,13 @@ display_daily_totals(pointers_t * pointers)
 	}
       else
 	fprintf(html, "<HR>");
-      if (!pointers->config->no_html_headers)
-	{
+      if (!pointers->config->no_html_headers) {
 	  fprintf(html, "<TITLE>xferstats: %s to %s</TITLE><BODY>\n",
 		  pointers->low_date, pointers->high_date);
+          if (pointers->config->refresh)
+            fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+                    pointers->config->refresh);
+
 	}
     }
   else
@@ -263,11 +270,13 @@ display_daily_totals(pointers_t * pointers)
     }
 
   count = 0;
-  if (pointers->config->daily_sort_pref == 2)
-    /* skip all of the entries before the last 'n' dictated by
-     * number_daily_stats */
-    if (pointers->config->number_daily_stats)
-      count = num_days - pointers->config->number_daily_stats + 1;
+  if (pointers->config->daily_sort_pref == 2) {
+	  /* skip all of the entries before the last 'n' dictated by
+	   * number_daily_stats */
+	  if (pointers->config->number_daily_stats)
+		  count = num_days - pointers->config->number_daily_stats + 1;
+	  if (count < 0) count = 0;
+  }
 
   for (; (current_day_ptr = pointers->first_day_ptr[count]); count++)
     {
@@ -332,6 +341,8 @@ display_daily_totals(pointers_t * pointers)
 			printf("Daily Transmission Statistics (%d day maximum)"
 			       "\n\n", pointers->config->number_daily_stats);
 		    }
+		  else
+		    putchar('\n');
 
 		  printf("                 Number Of    Number of    Average   Percent Of  Percent Of\n");
 		  printf("     Date        Files Sent  Bytes  Sent  Xmit Rate  Files Sent  Bytes Sent\n");
@@ -394,7 +405,7 @@ display_daily_totals(pointers_t * pointers)
 
 
 void
-display_misc_totals(pointers_t * pointers, int type)
+display_misc_totals(pointers_t * pointers, int type, char * htmlname)
 {
   xfmisc_t * current_path_ptr;
   void ** pointers_ptr = NULL;
@@ -506,79 +517,24 @@ display_misc_totals(pointers_t * pointers, int type)
 		    }
 		  else
 		    {
-		      switch(type)
+		      g_snprintf(filename, sizeof(filename) - 1,
+				 "%s%s",
+				 pointers->config->html_location ?
+				 pointers->config->html_location : "",
+				 htmlname);
+		      if (!(html = fopen(filename, "w")))
 			{
-			case T_DIR:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%sdir.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
-			case T_FILE:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%sfile.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
-			case T_TLD:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%stld.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
-			case T_HOST:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%shost.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
-			case T_DOMAIN:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%sdomain.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
-			case T_SIZE:
-			  g_snprintf(filename, sizeof(filename) - 1,
-				     "%ssize.html",
-				     pointers->config->html_location ?
-				     pointers->config->html_location : "");
-			  if (!(html = fopen(filename, "w")))
-			    {
-			      perror("fopen");
-			      exit(1);
-			    }
-			  break;
+			  perror("fopen");
+			  exit(1);
 			}
-		      if (!pointers->config->no_html_headers)
+		      if (!pointers->config->no_html_headers) {
 			fprintf(html, "<HTML><TITLE>xferstats: %s to %s"
 				"</TITLE><BODY>", pointers->low_date,
 				pointers->high_date);
+ 		        if (pointers->config->refresh)
+		          fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+		                  pointers->config->refresh);
+		      }
 		    }
 		}
 	      
@@ -1044,9 +1000,13 @@ display_hourly_totals(pointers_t * pointers)
 	      perror("fopen");
 	      exit(1);
 	    }
-	  if (!pointers->config->no_html_headers)
+	  if (!pointers->config->no_html_headers) {
 	    fprintf(html, "<HTML><TITLE>xferstats: %s to %s</TITLE><BODY>\n",
 		    pointers->low_date, pointers->high_date);
+            if (pointers->config->refresh)
+              fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+                      pointers->config->refresh);
+          }
 	}
       else
 	{
@@ -1156,9 +1116,13 @@ display_monthly_totals(pointers_t * pointers)
 	      perror("fopen");
 	      exit(1);
 	    }
-	  if (!pointers->config->no_html_headers)
+	  if (!pointers->config->no_html_headers) {
 	    fprintf(html, "<HTML><TITLE>xferstats: %s to %s</TITLE><BODY>\n",
 		    pointers->low_date, pointers->high_date);
+            if (pointers->config->refresh)
+              fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+                      pointers->config->refresh);
+	  }
 	}
       else
 	{
@@ -1269,9 +1233,13 @@ display_dow_totals(pointers_t * pointers)
 	      perror("fopen");
 	      exit(1);
 	    }
-	  if (!pointers->config->no_html_headers)
+	  if (!pointers->config->no_html_headers) {
 	    fprintf(html, "<HTML><TITLE>xferstats: %s to %s</TITLE><BODY>\n",
 		    pointers->low_date, pointers->high_date);
+            if (pointers->config->refresh)
+              fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+                      pointers->config->refresh);
+	  }
 	}
       else
 	{
@@ -1382,9 +1350,13 @@ display_dom_totals(pointers_t * pointers)
 	      perror("fopen");
 	      exit(1);
 	    }
-	  if (!pointers->config->no_html_headers)
+	  if (!pointers->config->no_html_headers) {
 	    fprintf(html, "<HTML><TITLE>xferstats: %s to %s</TITLE><BODY>\n",
 		    pointers->low_date, pointers->high_date);
+            if (pointers->config->refresh)
+              fprintf(html, "<META HTTP-EQUIV=\"Refresh\" CONTENT=%d>\n",
+                      pointers->config->refresh);
+	  }
 	}
       else
 	{
