@@ -1,5 +1,5 @@
 /*
- * Copyright 1997, 1998 Phil Schwan <pschwan@cmu.edu>
+ * Copyright 1997-1999 Phil Schwan <pschwan@cmu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #endif
 
 #ifdef HAVE_STRING_H
-#  include <string.h> /* needed everywhere */
+#  include <string.h>
 #endif
 
 #ifdef HAVE_MMAP
@@ -54,8 +54,7 @@
 #include <stdlib.h>
 #include "xferstats.h"
 
-const char MONTHS[12][3] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
-			    "Aug", "Sep", "Oct", "Nov", "Dec"};
+extern char MONTHS[12][4];
 
 int
 dir_in_list(GSList * list, char * path)
@@ -74,19 +73,21 @@ dir_in_list(GSList * list, char * path)
 } /* dir_in_list */
 
 
-void
+int
 parse_wuftpd_logfile(pointers_t * pointers)
 {
-  ftp_entry_t * ftp_line, * current_ftp_ptr = NULL;
-  char foo[2048], tmp_host[2048], tmp_path[2048], user_type, in_out, * logfile;
-  int len;
-  GSList * clist;
+  static ftp_entry_t * ftp_line, * current_ftp_ptr = NULL;
+  static char foo[2048], tmp_host[2048], tmp_path[2048], user_type, in_out,
+    * logfile;
+  static int len;
+  static GSList * clist = NULL;
 #ifdef HAVE_MMAP
-  struct stat stat_buf;
-  int file_fd = -1, size;
-  char * map = NULL, * map_curr = NULL, * a, * b;
+  static struct stat stat_buf;
+  static int file_fd = -1, size;
+  static char * map = NULL, * map_curr = NULL, * a, * b;
+  int lines = 0;
 #else /* HAVE_MMAP */
-  FILE * log_stream = NULL;
+  static FILE * log_stream = NULL;
 #endif /* HAVE_MMAP */
 
 #ifdef DEBUGS
@@ -94,281 +95,296 @@ parse_wuftpd_logfile(pointers_t * pointers)
 	  (long) time(NULL));
 #endif
 
-  for (clist = pointers->config->logfiles; clist; clist = clist->next)
-    {
-      logfile = clist->data;
+  if (!clist) /* in theory this is only true once */
+    clist = pointers->config->logfiles;
+
+  if (pointers->first_ftp_line) {
+    for (ftp_line = pointers->first_ftp_line; ftp_line; ) {
+      current_ftp_ptr = ftp_line->next_ptr;
+      free(ftp_line->host);
+      free(ftp_line->path);
+      free(ftp_line);
+      ftp_line = current_ftp_ptr;
+    }
+    pointers->first_ftp_line = NULL;
+  }
+
+  for (; clist; clist = clist->next) {
+    logfile = clist->data;
 
 #ifdef HAVE_MMAP
-      if (!pointers->config->use_stdin)
-	{
-	  if ((file_fd = open(logfile, O_RDONLY)) < 0)
-	    {
-	      fprintf(stderr, "xferstats: fatal: file \"%s\" not found.\n",
-		      logfile);
-	      exit(1);
-	    }
-	  stat(logfile, &stat_buf);
+    if (!pointers->config->use_stdin  && !map) {
+      if ((file_fd = open(logfile, O_RDONLY)) < 0) {
+	fprintf(stderr, "xferstats: fatal: file \"%s\" not found.\n",
+		logfile);
+	exit(1);
+      }
+      stat(logfile, &stat_buf);
 #  ifdef _AIX
-	  /* Only AIX, it seems, wants this mmap typecast to int... */
-	  if ((int)(map_curr = map =
-		    (char *)mmap(NULL, stat_buf.st_size, PROT_READ, MAP_SHARED,
-				 file_fd, 0)) < 0)
-	    {
-	      perror("mmap");
-	      exit(1);
-	    }
+      /* Only AIX, it seems, wants this mmap typecast to int... */
+      if ((int)(map_curr = map =
+		(char *)mmap(NULL, stat_buf.st_size, PROT_READ, MAP_SHARED,
+			     file_fd, 0)) < 0) {
+	perror("mmap");
+	exit(1);
+      }
 #  else /* _AIX */
-	  /* Linux doesn't care, and Solaris segfaults if it's an int.  God how
-	   * I love Linux... */
-	  if ((map_curr = map =
-	       (char *)mmap(NULL, stat_buf.st_size, PROT_READ, MAP_SHARED,
-			    file_fd, 0)) < 0)
-	    {
-	      perror("mmap");
-	      exit(1);
-	    }
+      /* Linux doesn't care, and Solaris segfaults if it's an int.  God how
+       * I love Linux... */
+      if ((map_curr = map =
+	   (char *)mmap(NULL, stat_buf.st_size, PROT_READ,
+			MAP_SHARED, file_fd, 0)) < 0) {
+	perror("mmap");
+	exit(1);
+      }
 #  endif /* _AIX */
-	}
+    }
 #else /* HAVE_MMAP */
-      if (!pointers->config->use_stdin)
-	if (!(log_stream = fopen(logfile, "r")))
-	  {
-	    perror("parse_wuftpd_logfile: fopen");
-	    exit(1);
-	  }
+    if (!pointers->config->use_stdin && !log_stream)
+      if (!(log_stream = fopen(logfile, "r"))) {
+	perror("parse_wuftpd_logfile: fopen");
+	exit(1);
+      }
 #endif /* HAVE_MMAP */
-
-      while (1)
-	{
+    
+    while (1) {
 #ifdef HAVE_MMAP
-	  if (pointers->config->use_stdin)
-	    {
-	      if (feof(stdin))
-		break;
-	      /* there's probably a better way to do this :) */
-	      fgets(foo, sizeof(foo), stdin);
-	    }
-	  else
-	    {
-	      if (map_curr - map >= stat_buf.st_size)
-		{
+      if (pointers->config->use_stdin) {
+	if (feof(stdin))
+	  break;
+	/* there's probably a better way to do this :) */
+	fgets(foo, sizeof(foo), stdin);
+      } else {
+	if (map_curr - map >= stat_buf.st_size) {
 #ifdef DEBUG3
-		  fprintf(stderr, "*** Reached the end of the map ***\n");
+	  fprintf(stderr, "*** Reached the end of the map ***\n");
 #endif
-		  munmap(map, stat_buf.st_size);
-		  close(file_fd);
-		  break;
-		}
+	  munmap(map, stat_buf.st_size);
+	  close(file_fd);
+	  map = NULL;
+	  break;
+	}
 
-	      a = memchr(map_curr, '\n', (size = stat_buf.st_size - (map_curr -
-								     map)));
-	      if (a)
-		b = memchr(map_curr, '\0', a - map_curr);
-	      else
-		b = memchr(map_curr, '\0', size);
-
-	      if (a && b)
-		{
-		  if (a < b)
-		    len = a - map_curr;
-		  else
-		    len = b - map_curr;
-		}
-	      else if (a)
-		len = a - map_curr;
-	      else if (b)
-		len = b - map_curr;
-	      else
-		len = size;
-
-	      memcpy(foo, map_curr,
-		     len > sizeof(foo) - 1 ? (len = sizeof(foo) - 1) : len);
-	      foo[len] = '\0';	
-	      map_curr += len + 1;
-	    }
+	a = memchr(map_curr, '\n', (size = stat_buf.st_size - (map_curr -
+							       map)));
+	if (a)
+	  b = memchr(map_curr, '\0', a - map_curr);
+	else
+	  b = memchr(map_curr, '\0', size);
+	
+	if (a && b) {
+	  if (a < b)
+	    len = a - map_curr;
+	  else
+	    len = b - map_curr;
+	} else if (a)
+	  len = a - map_curr;
+	else if (b)
+	  len = b - map_curr;
+	else
+	  len = size;
+	
+	memcpy(foo, map_curr,
+	       len > sizeof(foo) - 1 ? (len = sizeof(foo) - 1) : len);
+	foo[len] = '\0';	
+	map_curr += len + 1;
+      }
 #else /* HAVE_MMAP */
-	  if (pointers->config->use_stdin)
-	    {
-	      if (feof(stdin))
-		break;
-	      /* there's probably a better way to do this :) */
-	      fgets(foo, sizeof(foo), stdin);
-	    }
-          else
-	    {
-	      if (feof(log_stream))
-		break;
-
-	      fgets(foo, sizeof(foo), log_stream);
-	    }
+      if (pointers->config->use_stdin) {
+	if (feof(stdin))
+	  break;
+	/* there's probably a better way to do this :) */
+	fgets(foo, sizeof(foo), stdin);
+      } else {
+	if (feof(log_stream))
+	  break;
+	
+	fgets(foo, sizeof(foo), log_stream);
+      }
 #endif /* HAVE_MMAP */
-
-	  if ((len = strlen(foo)) < 42)
-	    continue; /* data is too small to be valid */
+      
+      if ((len = strlen(foo)) < 42)
+	continue; /* data is too small to be valid */
 #ifdef DEBUG3
-	  fprintf(stderr, "RAW READ: %s\n", foo);
+      fprintf(stderr, "RAW READ: %s\n", foo);
 #endif
-	  ftp_line = G_MEM_CHUNK_ALLOC(pointers->ftpentry_chunk);
-
-	  ftp_line->date[24] = '\0';
-	  memcpy(ftp_line->date, foo, 24);
-	  ftp_line->seconds = -1;
-	  ftp_line->data = 0;
-	  ftp_line->next_ptr = NULL;
-	  if ((foo[len - 1] == 'c' || foo[len - 1] == 'i') &&
-	      foo[len - 2] == ' ')
-	    /* this check isn't foolproof, but it's probably good enough for
-	     * now.  if it succeeds, this xferlog has a complete/incomplete
-	     * byte at the end of the line */
-	    sscanf(foo + 25, "%ld %s %lu %s %*c %*s %c %c %*s %*s %*d %*s %c",
-		   &ftp_line->seconds, tmp_host, &ftp_line->data, tmp_path,
-		   &in_out, &user_type, &ftp_line->complete);
-	  else
-	    {
-	      sscanf(foo + 25, "%ld %s %lu %s %*c %*s %c %c",
-		     &ftp_line->seconds, tmp_host, &ftp_line->data, tmp_path,
-		     &in_out, &user_type);
-	      ftp_line->complete = 'c';
-	    }
-
-	  if (ftp_line->date[0] == '\0' || tmp_host[0] == '\0' ||
-	      tmp_path[0] == '\0' || ftp_line->data == 0 ||
-	      ftp_line->seconds < 0 ||
-	      (user_type == 'a' && !pointers->config->anon_traffic) ||
-	      (user_type == 'g' && !pointers->config->guest_traffic) ||
-	      (user_type == 'r' && !pointers->config->real_traffic) ||
-	      (in_out == 'i' && !pointers->config->inbound) ||
-	      (in_out == 'o' && !pointers->config->outbound) ||
-	      (pointers->config->only_dir &&
-	       !dir_in_list(pointers->config->only_dir, tmp_path)))
-	    {
+      ftp_line = G_MEM_CHUNK_ALLOC(pointers->ftpentry_chunk);
+      
+      ftp_line->date[24] = '\0';
+      memcpy(ftp_line->date, foo, 24);
+      ftp_line->seconds = -1;
+      ftp_line->data = 0;
+      ftp_line->next_ptr = NULL;
+      if ((foo[len - 1] == 'c' || foo[len - 1] == 'i') &&
+	  foo[len - 2] == ' ')
+	/* this check isn't foolproof, but it's probably good enough for
+	 * now.  if it succeeds, this xferlog has a complete/incomplete
+	 * byte at the end of the line */
+	sscanf(foo + 25, "%ld %s %lu %s %*c %*s %c %c %*s %*s %*d %*s %c",
+	       &ftp_line->seconds, tmp_host, &ftp_line->data, tmp_path,
+	       &in_out, &user_type, &ftp_line->complete);
+      else {
+	sscanf(foo + 25, "%ld %s %lu %s %*c %*s %c %c",
+	       &ftp_line->seconds, tmp_host, &ftp_line->data, tmp_path,
+	       &in_out, &user_type);
+	ftp_line->complete = 'c';
+      }
+      
+      if (ftp_line->date[0] == '\0' || tmp_host[0] == '\0' ||
+	  tmp_path[0] == '\0' || ftp_line->data == 0 ||
+	  ftp_line->seconds < 0 ||
+	  (user_type == 'a' && !pointers->config->anon_traffic) ||
+	  (user_type == 'g' && !pointers->config->guest_traffic) ||
+	  (user_type == 'r' && !pointers->config->real_traffic) ||
+	  (in_out == 'i' && !pointers->config->inbound) ||
+	  (in_out == 'o' && !pointers->config->outbound) ||
+	  (pointers->config->only_dir &&
+	   !dir_in_list(pointers->config->only_dir, tmp_path))) {
 #ifdef DEBUG3
-	      if (ftp_line->date[0] == '\0')
-		fprintf(stderr, "Discarding -- no date.\n");
-	      else if (tmp_host[0] == '\0')
-		fprintf(stderr, "Discarding -- no host.\n");
-	      else if (tmp_path[0] == '\0')
-		fprintf(stderr, "Discarding -- no path.\n");
-	      else if (ftp_line->data == 0)
-		fprintf(stderr, "Discarding -- no data.\n");
-	      else if (ftp_line->seconds < 0)
-		fprintf(stderr, "Discarding -- no seconds.\n");
-	      else if (user_type == 'a' && !pointers->config->anon_traffic)
-		fprintf(stderr, "Discarding -- don't want anon traffic\n");
-	      else if (user_type == 'r' && !pointers->config->real_traffic)
-		fprintf(stderr, "Discarding -- don't want real traffic\n");
-	      else if (user_type == 'g' && !pointers->config->guest_traffic)
-		fprintf(stderr, "Discarding -- don't want guest traffic\n");
-	      else if (in_out == 'i' && !pointers->config->inbound)
-		fprintf(stderr, "Discarding -- don't want inbound traffic\n");
-	      else if (in_out == 'o' && !pointers->config->outbound)
-		fprintf(stderr, "Discarding -- don't want outbound traffic\n");
-	      else if (pointers->config->only_dir &&
-		       !dir_in_list(pointers->config->only_dir, tmp_path))
-		fprintf(stderr, "Discarding -- don't want this directory\n");
-	      else
-		fprintf(stderr, "Discarding -- no clue why.\n");
+	if (ftp_line->date[0] == '\0')
+	  fprintf(stderr, "Discarding -- no date.\n");
+	else if (tmp_host[0] == '\0')
+	  fprintf(stderr, "Discarding -- no host.\n");
+	else if (tmp_path[0] == '\0')
+	  fprintf(stderr, "Discarding -- no path.\n");
+	else if (ftp_line->data == 0)
+	  fprintf(stderr, "Discarding -- no data.\n");
+	else if (ftp_line->seconds < 0)
+	  fprintf(stderr, "Discarding -- no seconds.\n");
+	else if (user_type == 'a' && !pointers->config->anon_traffic)
+	  fprintf(stderr, "Discarding -- don't want anon traffic\n");
+	else if (user_type == 'r' && !pointers->config->real_traffic)
+	  fprintf(stderr, "Discarding -- don't want real traffic\n");
+	else if (user_type == 'g' && !pointers->config->guest_traffic)
+	  fprintf(stderr, "Discarding -- don't want guest traffic\n");
+	else if (in_out == 'i' && !pointers->config->inbound)
+	  fprintf(stderr, "Discarding -- don't want inbound traffic\n");
+	else if (in_out == 'o' && !pointers->config->outbound)
+	  fprintf(stderr, "Discarding -- don't want outbound traffic\n");
+	else if (pointers->config->only_dir &&
+		 !dir_in_list(pointers->config->only_dir, tmp_path))
+	  fprintf(stderr, "Discarding -- don't want this directory\n");
+	else
+	  fprintf(stderr, "Discarding -- no clue why.\n");
 #endif
 #ifndef SMFS
-	      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+	G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
 #endif
-	      continue; /* the line is invalid */
-	    }
-
-	  if (pointers->config->strict_check)
-	    /* These checks can significantly slow processing down, so they're
-	     * highly optional. */
-	    if (ftp_line->date[3] != ' ' || ftp_line->date[7] != ' ' ||
-		ftp_line->date[10] != ' ' || ftp_line->date[19] != ' ' ||
-		ftp_line->date[13] != ':' || ftp_line->date[16] != ':' ||
-		foo[24] != ' ' || !isdigit(ftp_line->date[9]) ||
-		!isdigit(ftp_line->date[11]) || !isdigit(ftp_line->date[12]) ||
-		!isdigit(ftp_line->date[14]) || !isdigit(ftp_line->date[15]) ||
-		!isdigit(ftp_line->date[17]) || !isdigit(ftp_line->date[18]) ||
-		!isdigit(ftp_line->date[20]) || !isdigit(ftp_line->date[21]) ||
-		!isdigit(ftp_line->date[22]) || !isdigit(ftp_line->date[23]) ||
-		!(ftp_line->date[8] == ' ' || ftp_line->date[8] == '1' ||
-		  ftp_line->date[8] == '2' || ftp_line->date[8] == '3'))
-	      {
+	continue; /* the line is invalid */
+      }
+      
+      if (pointers->config->strict_check)
+	/* These checks can significantly slow processing down, so they're
+	 * highly optional. */
+	if (ftp_line->date[3] != ' ' || ftp_line->date[7] != ' ' ||
+	    ftp_line->date[10] != ' ' || ftp_line->date[19] != ' ' ||
+	    ftp_line->date[13] != ':' || ftp_line->date[16] != ':' ||
+	    foo[24] != ' ' || !isdigit(ftp_line->date[9]) ||
+	    !isdigit(ftp_line->date[11]) || !isdigit(ftp_line->date[12]) ||
+	    !isdigit(ftp_line->date[14]) || !isdigit(ftp_line->date[15]) ||
+	    !isdigit(ftp_line->date[17]) || !isdigit(ftp_line->date[18]) ||
+	    !isdigit(ftp_line->date[20]) || !isdigit(ftp_line->date[21]) ||
+	    !isdigit(ftp_line->date[22]) || !isdigit(ftp_line->date[23]) ||
+	    !(ftp_line->date[8] == ' ' || ftp_line->date[8] == '1' ||
+	      ftp_line->date[8] == '2' || ftp_line->date[8] == '3')) {
 #  ifdef DEBUGS
-		fprintf(stderr, "*** Corrupt line discarded.\n");
+	  fprintf(stderr, "*** Corrupt line discarded.\n");
 #  endif
 #  ifndef SMFS
-		G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+	  G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
 #  endif
-		continue; /* the line is invalid */
-	      }
-
-	  if (!ftp_line->seconds)
-	    ftp_line->seconds = 1;
-
-	  ftp_line->host = g_malloc((len = strlen(tmp_host)) > MAXHOSTNAMELEN ?
-				   (len = MAXHOSTNAMELEN + 1) : ++len);
-	  memcpy(ftp_line->host, tmp_host, len - 1);
-	  ftp_line->host[len - 1] = '\0';
-
-	  if (pointers->config->strip_prefix_len &&
-	      !strncmp(tmp_path, pointers->config->strip_prefix,
-		       pointers->config->strip_prefix_len))
-	    {
-	      ftp_line->path = g_malloc((len = strlen(tmp_path) -
-					pointers->config->strip_prefix_len) >
-				       MAXPATHLEN ?
-				       (len = MAXPATHLEN + 1) : ++len);
-	      memcpy(ftp_line->path,
-		     tmp_path + pointers->config->strip_prefix_len, len - 1);
-	    }
-	  else
-	    {
-	      ftp_line->path = g_malloc((len = strlen(tmp_path)) > MAXPATHLEN ?
-				       (len = MAXPATHLEN + 1) : ++len);
-	      memcpy(ftp_line->path, tmp_path, len - 1);
-	    }
-	  ftp_line->path[len - 1] = '\0';
-      
-	  if (!pointers->first_ftp_line)
-	    {
-	      pointers->first_ftp_line = current_ftp_ptr = ftp_line;
-#ifdef DEBUG3
-	      fprintf(stderr, "*** First ftp_line structure created (%p)\n",
-		      ftp_line);
-#endif
-	    }
-	  else
-	    {
-	      current_ftp_ptr->next_ptr = ftp_line;
-	      current_ftp_ptr = ftp_line;
-#ifdef DEBUG3
-	      fprintf(stderr, "*** New ftp_line structure added (%p)\n",
-		      ftp_line);
-#endif
-	    }
-#ifdef DEBUG3
-	  fprintf(stderr, "%s %ld %s %lu %s %c %c\n", ftp_line->date,
-		  ftp_line->seconds, ftp_line->host, ftp_line->data,
-		  ftp_line->path, in_out, user_type);
-#endif
+	  continue; /* the line is invalid */
 	}
-
-#ifndef HAVE_MMAP
-      if (!pointers->config->use_stdin)
-	fclose(log_stream);
-#endif /* HAVE_MMAP */
+      
+      if ((pointers->config->begin_date[0] &&
+	   compare_dates(ftp_line->date,
+			 pointers->config->begin_date) == -1) ||
+	  (pointers->config->end_date[0] &&
+	   compare_dates(ftp_line->date,
+			 pointers->config->end_date) == 1)) {
+#ifdef DEBUG3
+	fprintf(stderr, "Discarding -- not in date range.\n");
+#endif
+#  ifndef SMFS
+	G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+#  endif
+	continue; /* the line is invalid */
+      }
+      
+      if (!ftp_line->seconds)
+	ftp_line->seconds = 1;
+      
+      ftp_line->host = g_malloc((len = strlen(tmp_host)) > MAXHOSTNAMELEN ?
+				(len = MAXHOSTNAMELEN + 1) : ++len);
+      memcpy(ftp_line->host, tmp_host, len - 1);
+      ftp_line->host[len - 1] = '\0';
+      
+      if (pointers->config->strip_prefix_len &&
+	  !strncmp(tmp_path, pointers->config->strip_prefix,
+		   pointers->config->strip_prefix_len)) {
+	ftp_line->path = g_malloc((len = strlen(tmp_path) -
+				   pointers->config->strip_prefix_len) >
+				  MAXPATHLEN ?
+				  (len = MAXPATHLEN + 1) : ++len);
+	memcpy(ftp_line->path,
+	       tmp_path + pointers->config->strip_prefix_len, len - 1);
+      } else {
+	ftp_line->path = g_malloc((len = strlen(tmp_path)) > MAXPATHLEN ?
+				  (len = MAXPATHLEN + 1) : ++len);
+	memcpy(ftp_line->path, tmp_path, len - 1);
+      }
+      ftp_line->path[len - 1] = '\0';
+      
+      if (!pointers->first_ftp_line) {
+	pointers->first_ftp_line = current_ftp_ptr = ftp_line;
+#ifdef DEBUG3
+	fprintf(stderr, "*** First ftp_line structure created (%p)\n",
+		ftp_line);
+#endif
+      } else {
+	current_ftp_ptr->next_ptr = ftp_line;
+	current_ftp_ptr = ftp_line;
+#ifdef DEBUG3
+	fprintf(stderr, "*** New ftp_line structure added (%p)\n",
+		ftp_line);
+#endif
+      }
+#ifdef DEBUG3
+      fprintf(stderr, "%s %ld %s %lu %s %c %c\n", ftp_line->date,
+	      ftp_line->seconds, ftp_line->host, ftp_line->data,
+	      ftp_line->path, in_out, user_type);
+#endif
+      lines++;
+      
+      if (pointers->config->chunk_input &&
+	  lines >= pointers->config->chunk_input)
+	return lines;
     }
+    
+#ifndef HAVE_MMAP
+    if (!pointers->config->use_stdin) {
+      fclose(log_stream);
+      log_stream = NULL;
+    }
+#endif /* HAVE_MMAP */
+  }
 
 #ifdef DEBUGS
   fprintf(stderr, "parse_wuftpd_logfile finished. (%ld)\n", (long) time(NULL));
 #endif
+
+  return 0;
 } /* parse_wuftpd_logfile */
 
 
-void
+int
 parse_ncftpd_logfile(pointers_t * pointers)
 {
   ftp_entry_t * ftp_line, * current_ftp_ptr = NULL;
   char foo[2048], * tmp_char, tmp_host[2048], tmp_path[2048], tmp_date[4],
     user_name[2048], tmp_str[2048], * logfile;
   int temp, len;
+  int lines = 0;
   GSList * clist;
 #ifdef HAVE_MMAP
   struct stat stat_buf;
@@ -537,107 +553,127 @@ parse_ncftpd_logfile(pointers_t * pointers)
 		continue; /* the line is invalid */
 	      }
 
-	      tmp_char = foo + 32;
-	      while ((tmp_char = strchr(tmp_char, ',')) != NULL)
-		*tmp_char = ' ';
+	  tmp_char = foo + 32;
+	  while ((tmp_char = strchr(tmp_char, ',')) != NULL)
+	    *tmp_char = ' ';
 	      
-	      /* there's an extra %s in the next line to catch the decimal
-	       * portion of the 'seconds' */
-	      sscanf(foo + 32, "%s %lu %ld %s %s %s %s %s", tmp_path,
-		     &ftp_line->data, &ftp_line->seconds, tmp_str, tmp_str,
-		     user_name, tmp_str, tmp_host);
+	  /* there's an extra %s in the next line to catch the decimal
+	   * portion of the 'seconds' */
+	  sscanf(foo + 32, "%s %lu %ld %s %s %s %s %s", tmp_path,
+		 &ftp_line->data, &ftp_line->seconds, tmp_str, tmp_str,
+		 user_name, tmp_str, tmp_host);
 
-	      if ((foo[30] == 'R' && !pointers->config->outbound) ||
-		  (foo[30] != 'R' && !pointers->config->inbound) ||
-		  !strcmp(ftp_line->date, "") || ftp_line->seconds < 0 ||
-		  (pointers->config->only_dir &&
-		   !dir_in_list(pointers->config->only_dir, tmp_path)))
+	  if ((foo[30] == 'R' && !pointers->config->outbound) ||
+	      (foo[30] != 'R' && !pointers->config->inbound) ||
+	      !strcmp(ftp_line->date, "") || ftp_line->seconds < 0 ||
+	      (pointers->config->only_dir &&
+	       !dir_in_list(pointers->config->only_dir, tmp_path)))
+	    {
+#ifndef SMFS
+	      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+#endif
+	      continue; /* we don't want this log line */
+	    }
+	  if (!strcmp(user_name, "anonymous"))
+	    {
+	      if (!pointers->config->anon_traffic)
 		{
 #ifndef SMFS
 		  G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
 #endif
-		  continue; /* we don't want this log line */
+		  continue; /* we don't want anon traffic */
 		}
-	      if (!strcmp(user_name, "anonymous"))
+	    }
+	  else
+	    {
+	      if (!pointers->config->real_traffic)
 		{
-		  if (!pointers->config->anon_traffic)
-		    {
 #ifndef SMFS
-		      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+		  G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
 #endif
-		      continue; /* we don't want anon traffic */
-		    }
-		}
-	      else
-		{
-		  if (!pointers->config->real_traffic)
-		    {
-#ifndef SMFS
-		      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
-#endif
-		      continue; /* we don't want anon traffic */
-		    }
-		  /* for a real user, the log line (after comma parsing) looks
-		   * like:
-		   * /foo 12345 5.123 30.069 pschwan  127.0.0.1  OK
-		   * which means that the hostname is in tmp_str instead of
-		   * tmp_host so we must use it instead */
-		  memcpy(tmp_host, tmp_str, MAXHOSTNAMELEN);
+		  continue; /* we don't want anon traffic */
 		}
+	      /* for a real user, the log line (after comma parsing) looks
+	       * like:
+	       * /foo 12345 5.123 30.069 pschwan  127.0.0.1  OK
+	       * which means that the hostname is in tmp_str instead of
+	       * tmp_host so we must use it instead */
+	      memcpy(tmp_host, tmp_str, MAXHOSTNAMELEN);
+	    }
 	      
-	      if (!ftp_line->seconds)
-		ftp_line->seconds = 1;
-
-	      ftp_line->host =
-		g_malloc((len = strlen(tmp_host)) > MAXHOSTNAMELEN ?
-			(len = MAXHOSTNAMELEN + 1) : ++len);
-	      memcpy(ftp_line->host, tmp_host, len - 1);
-	      ftp_line->host[len - 1] = '\0';
-
-	      if (!strncmp(tmp_path, pointers->config->strip_prefix,
-			   pointers->config->strip_prefix_len))
-		{
-		  ftp_line->path =
-		    g_malloc((len = strlen(tmp_path) -
-			      pointers->config->strip_prefix_len)
-			     > MAXPATHLEN ? (len = MAXPATHLEN + 1) : ++len);
-		  memcpy(ftp_line->path,
-			 tmp_path + pointers->config->strip_prefix_len,
-			 len - 1);
-		}
-	      else
-		{
-		  ftp_line->path = g_malloc((len = strlen(tmp_path)) >
-					    MAXPATHLEN ? (len = MAXPATHLEN + 1)
-					    : ++len);
-		  memcpy(ftp_line->path, tmp_path, len - 1);
-		}
-	      ftp_line->path[len - 1] = '\0';
-      
-	      if (!pointers->first_ftp_line)
-		{
-		  pointers->first_ftp_line = current_ftp_ptr = ftp_line;
-#ifdef DEBUG
-		  fprintf(stderr, "*** First ftp_line structure created (%p)"
-			  "\n", ftp_line);
+	  if ((pointers->config->begin_date[0] &&
+	       compare_dates(ftp_line->date,
+			     pointers->config->begin_date) == -1) ||
+	      (pointers->config->end_date[0] &&
+	       compare_dates(ftp_line->date,
+			     pointers->config->end_date) == 1))
+	    {
+#ifdef DEBUG3
+	      fprintf(stderr, "Discarding -- not in date range.\n");
 #endif
-		}
-	      else
-		{
-		  while (current_ftp_ptr->next_ptr != NULL)		
-		    current_ftp_ptr = current_ftp_ptr->next_ptr;
-		  current_ftp_ptr->next_ptr = ftp_line;
+#  ifndef SMFS
+	      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+#  endif
+	      continue; /* the line is invalid */
+	    }
+
+	  if (!ftp_line->seconds)
+	    ftp_line->seconds = 1;
+
+	  ftp_line->host =
+	    g_malloc((len = strlen(tmp_host)) > MAXHOSTNAMELEN ?
+		     (len = MAXHOSTNAMELEN + 1) : ++len);
+	  memcpy(ftp_line->host, tmp_host, len - 1);
+	  ftp_line->host[len - 1] = '\0';
+	  
+	  if (!strncmp(tmp_path, pointers->config->strip_prefix,
+		       pointers->config->strip_prefix_len))
+	    {
+	      ftp_line->path =
+		g_malloc((len = strlen(tmp_path) -
+			  pointers->config->strip_prefix_len)
+			 > MAXPATHLEN ? (len = MAXPATHLEN + 1) : ++len);
+	      memcpy(ftp_line->path,
+		     tmp_path + pointers->config->strip_prefix_len, len - 1);
+	    }
+	  else
+	    {
+	      ftp_line->path = g_malloc((len = strlen(tmp_path)) >
+					MAXPATHLEN ? (len = MAXPATHLEN + 1)
+					: ++len);
+	      memcpy(ftp_line->path, tmp_path, len - 1);
+	    }
+	  ftp_line->path[len - 1] = '\0';
+	  
+	  if (!pointers->first_ftp_line)
+	    {
+	      pointers->first_ftp_line = current_ftp_ptr = ftp_line;
 #ifdef DEBUG
-		  fprintf(stderr, "*** New ftp_line structure added (%p)\n",
-			  ftp_line);
-#endif
-		}
-#ifdef DEBUG2
-	      fprintf(stderr, "%s %ld %s %lu %s\n", ftp_line->date,
-		      ftp_line->seconds, ftp_line->host, ftp_line->data,
-		      ftp_line->path);
+	      fprintf(stderr, "*** First ftp_line structure created (%p)\n"
+		      ftp_line);
 #endif
 	    }
+	  else
+	    {
+	      while (current_ftp_ptr->next_ptr != NULL)		
+		current_ftp_ptr = current_ftp_ptr->next_ptr;
+	      current_ftp_ptr->next_ptr = ftp_line;
+#ifdef DEBUG
+	      fprintf(stderr, "*** New ftp_line structure added (%p)\n",
+		      ftp_line);
+#endif
+	    }
+#ifdef DEBUG2
+	  fprintf(stderr, "%s %ld %s %lu %s\n", ftp_line->date,
+		  ftp_line->seconds, ftp_line->host, ftp_line->data,
+		  ftp_line->path);
+#endif
+	    }
+      lines++;
+      
+      if (pointers->config->chunk_input &&
+	  lines >= pointers->config->chunk_input)
+	return lines;
 	}
 
 #ifndef HAVE_MMAP
@@ -649,15 +685,18 @@ parse_ncftpd_logfile(pointers_t * pointers)
 #ifdef DEBUGS
   fprintf(stderr, "parse_ncftpd_logfile finished.\n");
 #endif
+
+  return 0;
 } /* parse_ncftpd_logfile */
 
 
-void
+int
 parse_apache_logfile(pointers_t * pointers)
 {
   ftp_entry_t * ftp_line, * current_ftp_ptr = NULL;
   char foo[2048], tmp_host[2048], tmp_path[2048], tmp_user[2048], * logfile;
   int len;
+  int lines = 0;
   GSList * clist;
 #ifdef HAVE_MMAP
   struct stat stat_buf;
@@ -830,6 +869,22 @@ parse_apache_logfile(pointers_t * pointers)
 		continue; /* the line is invalid */
 	      }
 
+	  if ((pointers->config->begin_date[0] &&
+	       compare_dates(ftp_line->date,
+			     pointers->config->begin_date) == -1) ||
+	      (pointers->config->end_date[0] &&
+	       compare_dates(ftp_line->date,
+			     pointers->config->end_date) == 1))
+	    {
+#ifdef DEBUG3
+	      fprintf(stderr, "Discarding -- not in date range.\n");
+#endif
+#  ifndef SMFS
+	      G_MEM_CHUNK_FREE(pointers->ftpentry_chunk, ftp_line);
+#  endif
+	      continue; /* the line is invalid */
+	    }
+
 	  if (!ftp_line->seconds)
 	    ftp_line->seconds = 1;
 	  
@@ -878,6 +933,11 @@ parse_apache_logfile(pointers_t * pointers)
 		  ftp_line->seconds, ftp_line->host, ftp_line->data,
 		  ftp_line->path);
 #endif
+      lines++;
+      
+      if (pointers->config->chunk_input &&
+	  lines >= pointers->config->chunk_input)
+	return lines;
 	}
 
 #ifndef HAVE_MMAP
@@ -889,4 +949,6 @@ parse_apache_logfile(pointers_t * pointers)
 #ifdef DEBUGS
   fprintf(stderr, "parse_apache_logfile finished. (%ld)\n", (long) time(NULL));
 #endif
+
+  return 0;
 } /* parse_apache_logfile */
