@@ -1,6 +1,6 @@
 /*
- * xferstats, a wu-ftpd and ncftpd logfile parser and report generator
- * Copyright 1997, 1998 Phil Schwan <pschwan@cmu.edu>
+ * xferstats, a logfile parser and report generator
+ * Copyright 1997-1999 Phil Schwan <pschwan@cmu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,13 +50,14 @@ const char LONG_DAYS[7][10] = {"Sunday", "Monday", "Tuesday", "Wednesday",
 const char LONG_MONTHS[12][10] = {"January", "February", "March", "April",
 				  "May", "June", "July", "August", "September",
 				  "October", "November", "December"};
+const char MONTHS[12][4] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
+			    "Aug", "Sep", "Oct", "Nov", "Dec"};
 /* don't ask ... */
 const char NUMBERS[31][3] = {"01", "02", "03", "04", "05", "06", "07", "08",
 			     "09", "10", "11", "12", "13", "14", "15", "16",
 			     "17", "18", "19", "20", "21", "22", "23", "24",
 			     "25", "26", "27", "28", "29", "30", "31"};
-const char T_NAMES[6][10] = {"file", "dir", "host", "TLD", "domain",
-			    "size"};
+const char T_NAMES[6][10] = {"file", "dir", "host", "TLD", "domain", "size"};
 
 typedef struct _GHashNode      GHashNode;
 
@@ -137,7 +138,7 @@ month2int(char *month)
  * 0 for identical days
  * 1 for date1 being more in the future than date2
  * -1 for date1 being more in the past than date2 */
-static inline int
+inline int
 compare_dates(char date1[25], char date2[25])
 {
   int dw1, dw2, len = strlen(date1);
@@ -392,12 +393,8 @@ generate_hourly_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       pointers->hour_data[index]->data += ftp_line->data;
     }
 
-  /* One probably wonders, why sort at all in the default case?  Won't they
-   * already be in numerical order?  The answer is, not necessarily, if this
-   * isn't the first time this function is being run. */
-
-  switch(pointers->config->hourly_sort_pref)
-    {
+  if (pointers->done) {
+    switch(pointers->config->hourly_sort_pref) {
     case 0:
       sort_func = (void *)xf_sort_by_data;
       break;
@@ -412,7 +409,8 @@ generate_hourly_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       break;
     }
 
-  qsort((void *)pointers->hour_data, 24, (size_t) sizeof(void *), sort_func);
+    qsort((void *)pointers->hour_data, 24, (size_t) sizeof(void *), sort_func);
+  }
 
 #if 0
   if (pointers->config->additive_db)
@@ -477,8 +475,8 @@ generate_monthly_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
 	  pointers->month_data[index]->data += ftp_line->data;
 	}
 
-  switch(pointers->config->monthly_sort_pref)
-    {
+  if (pointers->done) {
+    switch(pointers->config->monthly_sort_pref) {
     case 0:
       sort_func = (void *)xf_sort_by_data;
       break;
@@ -492,7 +490,8 @@ generate_monthly_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       sort_func = (void *)xf_sort_by_number;
     }
 
-  qsort((void *)pointers->month_data, 12, (size_t) sizeof(void *), sort_func);
+    qsort((void *)pointers->month_data, 12, (size_t)sizeof(void *), sort_func);
+  }
 
 #if 0
   if (pointers->config->additive_db)
@@ -531,7 +530,8 @@ generate_dom_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
   fprintf(stderr, "Beginning day-of-the-month data generation...\n");
 #endif
 
-  if (!pointers->config->number_daily_stats && day)
+  if (!pointers->config->number_daily_stats && day &&
+      !pointers->config->chunk_input)
     {
 #ifdef DEBUGS
       fprintf(stderr, "   Using previously generated daily data...\n");
@@ -580,8 +580,8 @@ generate_dom_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
 #endif
       }
 
-  switch(pointers->config->dom_sort_pref)
-    {
+  if (pointers->done) {
+    switch(pointers->config->dom_sort_pref) {
     case 0:
       sort_func = (void *)xf_sort_by_data;
       break;
@@ -595,7 +595,8 @@ generate_dom_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       sort_func = (void *)xf_sort_by_number;
     }
 
-  qsort((void *)pointers->dom_data, 31, (size_t) sizeof(void *), sort_func);
+    qsort((void *)pointers->dom_data, 31, (size_t) sizeof(void *), sort_func);
+  }
 
 #if 0
   if (pointers->config->additive_db)
@@ -634,7 +635,8 @@ generate_dow_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
   fprintf(stderr, "Beginning day-of-the-week data generation...\n");
 #endif
 
-  if (!pointers->config->number_daily_stats && day)
+  if (!pointers->config->number_daily_stats && day &&
+      !pointers->config->chunk_input)
     {
 #ifdef DEBUGS
       fprintf(stderr, "   Using previously generated daily data...\n");
@@ -667,9 +669,8 @@ generate_dow_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
 	fprintf(stderr, "Invalid day in date \"%s\"\n", day->name);
 #endif
 
-
-  switch(pointers->config->dow_sort_pref)
-    {
+  if (pointers->done) {
+    switch(pointers->config->dow_sort_pref) {
     case 0:
       sort_func = (void *)xf_sort_by_data;
       break;
@@ -680,7 +681,8 @@ generate_dow_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       sort_func = (void *)xf_sort_by_number;
     }
 
-  qsort((void *)pointers->weekday_data, 7, (size_t) sizeof(void *), sort_func);
+    qsort((void *)pointers->weekday_data, 7, (size_t)sizeof(void *),sort_func);
+  }
 
 #if 0
   if (pointers->config->additive_db)
@@ -712,7 +714,7 @@ generate_daily_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
   ftp_entry_t * ftp_line = first_ftp_line;
   xfmisc_t * current_day_ptr;
   char temp_date[16];
-  GHashTable * ht;
+  static GHashTable * ht = NULL;
   int count;
   int (*sort_func)(const void *, const void *) = NULL;
 
@@ -720,7 +722,8 @@ generate_daily_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
   fprintf(stderr, "Beginning daily data generation...\n");
 #endif
 
-  ht = g_hash_table_new(g_str_hash, g_str_equal);
+  if (!ht)
+    ht = g_hash_table_new(g_str_hash, g_str_equal);
   
   /* If [low, high]_date hasn't already been set (perhaps by the additive DB
    * code, or a (god forbid) previous execution of this function) set them to
@@ -747,11 +750,11 @@ generate_daily_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       xf_set_value(ht, temp_date, current_day_ptr, ftp_line, 1);
     }
 
-  pointers->first_day_ptr = (xfmisc_t **)xf_make_array(ht, &count);
-  g_hash_table_destroy(ht);
-
-  switch(pointers->config->daily_sort_pref)
-    {
+  if (pointers->done) {
+    pointers->first_day_ptr = (xfmisc_t **)xf_make_array(ht, &count);
+    g_hash_table_destroy(ht);
+  
+    switch(pointers->config->daily_sort_pref) {
     case 0:
       sort_func = (void *)xf_sort_by_data;
       break;
@@ -766,8 +769,10 @@ generate_daily_data(pointers_t * pointers, ftp_entry_t * first_ftp_line)
       break;
     }
 
-  qsort((void *)pointers->first_day_ptr, (size_t) count,
-	(size_t) sizeof(void *), sort_func);
+    qsort((void *)pointers->first_day_ptr, (size_t) count,
+	  (size_t) sizeof(void *), sort_func);
+  }
+
 #if 0
   if (pointers->config->additive_db)
     {
@@ -871,37 +876,59 @@ generate_misc_data(pointers_t *pointers, ftp_entry_t * first_ftp_line,
 	  (long) time(NULL));
 #endif
 
-  ht = g_hash_table_new(g_str_hash, g_str_equal);
-
-  switch (type)
-    {
-    case T_FILE:
-      pointers_ptr = pointers->first_file_ptr;
-      break;
-    case T_DIR:
-      pointers_ptr = pointers->first_dir_ptr;
-      break;
-    case T_HOST:
-      pointers_ptr = pointers->first_host_ptr;
-      break;
-    case T_TLD:
-      pointers_ptr = pointers->first_tld_ptr;
-      break;
-    case T_DOMAIN:
-      pointers_ptr = pointers->first_domain_ptr;
-      break;
-    case T_SIZE:
-      pointers_ptr = pointers->first_size_ptr;
-      break;
-    default:
-      fprintf(stderr, "generate_misc_data: invalid type.  if you get this, "
-	      "please mail\n pschwan@cmu.edu with the version number (%s).  "
-	      "This is important!\n", VERSION);
-      return;
-    }
+  switch (type) {
+  case T_FILE:
+    pointers_ptr = pointers->first_file_ptr;
+    if (!pointers->file_ht)
+      ht = pointers->file_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->file_ht;
+    break;
+  case T_DIR:
+    pointers_ptr = pointers->first_dir_ptr;
+    if (!pointers->dir_ht)
+      ht = pointers->dir_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->dir_ht;
+    break;
+  case T_HOST:
+    pointers_ptr = pointers->first_host_ptr;
+    if (!pointers->host_ht)
+      ht = pointers->host_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->host_ht;
+    break;
+  case T_TLD:
+    pointers_ptr = pointers->first_tld_ptr;
+    if (!pointers->tld_ht)
+      ht = pointers->tld_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->tld_ht;
+    break;
+  case T_DOMAIN:
+    pointers_ptr = pointers->first_domain_ptr;
+    if (!pointers->domain_ht)
+      ht = pointers->domain_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->domain_ht;
+    break;
+  case T_SIZE:
+    pointers_ptr = pointers->first_size_ptr;
+    if (!pointers->size_ht)
+      ht = pointers->size_ht = g_hash_table_new(g_str_hash, g_str_equal);
+    else
+      ht = pointers->size_ht;
+    break;
+  default:
+    fprintf(stderr, "generate_misc_data: invalid type.  if you get this, "
+	    "please mail\n pschwan@cmu.edu with the version number (%s).  "
+	    "This is important!\n", VERSION);
+    return;
+  }
 
   if (type == T_DOMAIN && pointers->config->host_traffic &&
-      pointers->first_host_ptr && !pointers->config->number_host_stats)
+      pointers->first_host_ptr && !pointers->config->number_host_stats &&
+      !pointers->config->chunk_input)
     {
       /* get domain data from host data--since it's already partly-compiled, it
        * saves time, as there will be fewer (or at the very least, the same
@@ -991,7 +1018,8 @@ generate_misc_data(pointers_t *pointers, ftp_entry_t * first_ftp_line,
 	}
     }
   else if (type == T_DIR && pointers->config->file_traffic &&
-	   pointers->first_file_ptr && !pointers->config->number_file_stats)
+	   pointers->first_file_ptr && !pointers->config->number_file_stats &&
+	   !pointers->config->chunk_input)
     {
       /* Take the dir data from the file data, for the same reason as above */
 
@@ -1037,7 +1065,8 @@ generate_misc_data(pointers_t *pointers, ftp_entry_t * first_ftp_line,
     }
   else if (type == T_TLD && pointers->config->domain_traffic &&
 	   pointers->first_domain_ptr &&
-	   !pointers->config->number_domain_stats)
+	   !pointers->config->number_domain_stats &&
+	   !pointers->config->chunk_input)
     {
       /* Take the TLD data from the domain data, for the same reason as the
        * two above blocks... */
@@ -1232,132 +1261,123 @@ generate_misc_data(pointers_t *pointers, ftp_entry_t * first_ftp_line,
 	}
     }
 
+  if (pointers->done) {
 #ifdef DEBUG
-  fprintf(stderr, "Starting to sort...\n");
+    fprintf(stderr, "Starting to sort...\n");
 #endif
 
-  /* ------ SORTING STUFF START HERE ------ */
+    /* In recognition of time wasted working on sorting only to replace it all
+     * with a basic libc function, I give you the following quote:
+     *
+     * "There are people out there who will do this, and they are very scary."
+     *     -- Alan Cox, "Space Aliens Ate My Mouse", Linux Expo, May 30 1998
+     */
 
-  /* In recognition of time wasted working on sorting only to replace it all
-   * with a basic libc function, I give you the following quote:
-   *
-   * "There are people out there who will do this, and they are very scary."
-   *     -- Alan Cox, "Space Aliens Ate My Mouse", Linux Expo, May 30 1998
-   */
-
-  array = xf_make_array(ht, &count);
-  g_hash_table_destroy(ht);
-
-  switch(type)
-    {
+    array = xf_make_array(ht, &count);
+    g_hash_table_destroy(ht);
+    
+    switch(type) {
     case T_FILE:
-      switch(pointers->config->file_sort_pref)
-	{
-	case 0:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	case 2:
-	  sort_func = (void *)xf_sort_alphabetical;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	}
+      switch(pointers->config->file_sort_pref) {
+      case 0:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      case 2:
+	sort_func = (void *)xf_sort_alphabetical;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      }
       break;
     case T_HOST:
-      switch(pointers->config->host_sort_pref)
-	{
-	case 1:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	case 2:
-	  sort_func = (void *)xf_sort_alphabetical;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	}
+      switch(pointers->config->host_sort_pref) {
+      case 1:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      case 2:
+	sort_func = (void *)xf_sort_alphabetical;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      }
       break;
     case T_DIR:
-      switch(pointers->config->dir_sort_pref)
-	{
-	case 1:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	case 2:
-	  sort_func = (void *)xf_sort_alphabetical;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	}
+      switch(pointers->config->dir_sort_pref) {
+      case 1:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      case 2:
+	sort_func = (void *)xf_sort_alphabetical;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      }
       break;
     case T_TLD:
-      switch(pointers->config->tld_sort_pref)
-	{
-	case 1:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	case 2:
-	  sort_func = (void *)xf_sort_alphabetical;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	}
+      switch(pointers->config->tld_sort_pref) {
+      case 1:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      case 2:
+	sort_func = (void *)xf_sort_alphabetical;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      }
       break;
     case T_DOMAIN:
-      switch(pointers->config->domain_sort_pref)
-	{
-	case 1:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	case 2:
-	  sort_func = (void *)xf_sort_alphabetical;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	}
+      switch(pointers->config->domain_sort_pref) {
+      case 1:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      case 2:
+	sort_func = (void *)xf_sort_alphabetical;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      }
       break;
     case T_SIZE:
-      switch(pointers->config->size_sort_pref)
-	{
-	case 0:
-	  sort_func = (void *)xf_sort_by_data;
-	  break;
-	case 1:
-	  sort_func = (void *)xf_sort_by_file_count;
-	  break;
-	case 3:
-	  sort_func = (void *)xf_sort_by_throughput;
-	  break;
-	default:
-	  sort_func = (void *)xf_sort_by_file_size;
-	  break;
-	}
+      switch(pointers->config->size_sort_pref) {
+      case 0:
+	sort_func = (void *)xf_sort_by_data;
+	break;
+      case 1:
+	sort_func = (void *)xf_sort_by_file_count;
+	break;
+      case 3:
+	sort_func = (void *)xf_sort_by_throughput;
+	break;
+      default:
+	sort_func = (void *)xf_sort_by_file_size;
+	break;
+      }
       break;
     }
 
-  qsort((void *)array, (size_t) count, (size_t) sizeof(void *), sort_func);
+    qsort((void *)array, (size_t) count, (size_t) sizeof(void *), sort_func);
 
-  switch(type)
-    {
+    switch(type) {
     case T_FILE:
       pointers->first_file_ptr = array;
       break;
@@ -1378,72 +1398,66 @@ generate_misc_data(pointers_t *pointers, ftp_entry_t * first_ftp_line,
       break;
     }
 
-  if (pointers->config->additive_db)
-    {
+    if (pointers->config->additive_db) {
       FILE * db;
 
-      switch(type)
-	{
-	case T_FILE:
-	  if (!(db = fopen("file.xdb", "w")))
-	    return;
-	  break;
-	case T_HOST:
-	  if (!(db = fopen("host.xdb", "w")))
-	    return;
-	  break;
-	case T_DIR:
-	  if (!(db = fopen("dir.xdb", "w")))
-	    return;
-	  break;
-	case T_TLD:
-	  if (!(db = fopen("tld.xdb", "w")))
-	    return;
-	  break;
-	case T_DOMAIN:
-	  if (!(db = fopen("domain.xdb", "w")))
-	    return;
-	  break;
-	case T_SIZE:
-	  if (!(db = fopen("size.xdb", "w")))
-	    return;
-	  break;
-	default:
+      switch(type) {
+      case T_FILE:
+	if (!(db = fopen("file.xdb", "w")))
+	  return;
+	break;
+      case T_HOST:
+	if (!(db = fopen("host.xdb", "w")))
+	  return;
+	break;
+      case T_DIR:
+	if (!(db = fopen("dir.xdb", "w")))
+	  return;
+	break;
+      case T_TLD:
+	if (!(db = fopen("tld.xdb", "w")))
+	  return;
+	break;
+      case T_DOMAIN:
+	if (!(db = fopen("domain.xdb", "w")))
+	  return;
+	break;
+      case T_SIZE:
+	if (!(db = fopen("size.xdb", "w")))
+	  return;
+	break;
+      default:
+	return;
+      }
+      
+      for (current_data_ptr = array[count = 0]; current_data_ptr;
+	   current_data_ptr = array[++count]) {
+	if (fwrite(current_data_ptr->name, sizeof(char),
+		   strlen(current_data_ptr->name) + 1, db) <
+	    strlen(current_data_ptr->name) + 1) {
+	  perror("generate_misc_data: fwrite(name)");
 	  return;
 	}
-
-      for (current_data_ptr = array[count = 0]; current_data_ptr;
-	   current_data_ptr = array[++count])
-	{
-	  if (fwrite(current_data_ptr->name, sizeof(char),
-		     strlen(current_data_ptr->name) + 1, db) <
-	      strlen(current_data_ptr->name) + 1)
-	    {
-	      perror("generate_misc_data: fwrite(name)");
-	      return;
-	    }
-	  if (fwrite(&current_data_ptr->data, sizeof(current_data_ptr->data),
-		     1, db) < 1)
-	    {
-	      perror("generate_misc_data: fwrite(data)");
-	      return;
-	    }
-	  if (fwrite(&current_data_ptr->file_count,
-		     sizeof(current_data_ptr->file_count), 1, db) < 1)
-	    {
-	      perror("generate_misc_data: fwrite(file_count)");
-	      return;
-	    }
-	  if (fwrite(&current_data_ptr->seconds,
-		     sizeof(current_data_ptr->seconds), 1, db) < 1)
-	    {
-	      perror("generate_misc_data: fwrite(seconds)");
-	      return;
-	    }
+	if (fwrite(&current_data_ptr->data, sizeof(current_data_ptr->data),
+		   1, db) < 1) {
+	  perror("generate_misc_data: fwrite(data)");
+	  return;
 	}
+	if (fwrite(&current_data_ptr->file_count,
+		   sizeof(current_data_ptr->file_count), 1, db) < 1) {
+	  perror("generate_misc_data: fwrite(file_count)");
+	  return;
+	}
+	if (fwrite(&current_data_ptr->seconds,
+		   sizeof(current_data_ptr->seconds), 1, db) < 1) {
+	  perror("generate_misc_data: fwrite(seconds)");
+	  return;
+	}
+      }
 
       fclose(db);
     }
+  }
 
 #ifdef DEBUGS
   fprintf(stderr, "%s data generation complete (%ld)\n", T_NAMES[type],
@@ -1769,6 +1783,10 @@ usage(char bad_arg)
 	 " -D <number>          depth of path detail for directories (default 3)\n"
 	 " +D <directory>       directory to report on, for example: +D /pub will\n"
 	 "                      report only on paths under /pub\n"
+	 " +B <timestamp>       the starting date for all reports (all prior transfers\n"
+	 "                      will be filtered).  Jan DD HH:MM:ss YYYY\n"
+	 " +E <timestamp>       the ending date, in the same format\n"
+	 " +P <prefix>		strip the specified prefix from pathnames\n"
 	 "\n"
 	 " -v, --version        display version information\n"
 	 " --help               this listing\n"
@@ -1839,7 +1857,7 @@ parse_cmdline(int argc, char *argv[], config_t * config)
 	  else if (!strcmp(argv[i]+2, "version"))
 	    {
 	      printf("%s\n", VERSION);
-	      printf("Copyright (C) 1997, 1998 Phil Schwan\n");
+	      printf("Copyright (C) 1997-1999 Phil Schwan\n");
 	      exit(0);
 	    }
 	}
@@ -1993,7 +2011,7 @@ parse_cmdline(int argc, char *argv[], config_t * config)
 		  continue;
 		case 'v':
                   printf("%s\n", VERSION);
-                  printf("Copyright (C) 1997, 1998 Phil Schwan\n");
+                  printf("Copyright (C) 1997-1999 Phil Schwan\n");
                   exit(0);
 		default:
 		  usage(argv[i][j]);
@@ -2092,11 +2110,7 @@ parse_cmdline(int argc, char *argv[], config_t * config)
 		    {
 		      item = g_malloc(sizeof(only_dir_t));
 
-		      if (!(item->dir = g_strdup(argv[i + 1])))
-			{
-			  perror("parse_cmdline: g_strdup");
-			  exit(0);
-			}
+		      item->dir = g_strdup(argv[i + 1]);
 		      item->len = strlen(argv[i + 1]);
 
 		      config->only_dir = g_slist_append(config->only_dir,
@@ -2110,6 +2124,107 @@ parse_cmdline(int argc, char *argv[], config_t * config)
 		    }
 		  j = strlen(argv[++i]);
 		  continue;
+		case 'P':
+		  if (i < (argc - 1) && *argv[i + 1] != '-' &&
+		      *argv[i + 1] != '+')
+		    {
+			/* FIXME: allow a list of stripped prefixes? */
+			if (config->strip_prefix != NULL)
+				g_free(config->strip_prefix);
+
+			config->strip_prefix =
+				g_malloc((config->strip_prefix_len =
+					  strlen(argv[i + 1])) + 1);
+			strcpy(config->strip_prefix, argv[i + 1]);
+		    }
+		  else
+		    {
+		      fprintf(stderr, "fatal: the +P parameter requires a "
+			      "string following it.\n");
+		      exit(1);
+		    }
+		  j = strlen(argv[++i]);
+		  continue;
+		case 'B':
+		  if (i < (argc - 1) && *argv[i + 1] != '-' &&
+		      *argv[i + 1] != '+')
+		    {
+		      char * bar = argv[i + 1];
+
+		      if (strlen(argv[i + 1]) != 20)
+			{
+			  fprintf(stderr, "fatal: the date must be in the "
+				  "Jan DD HH:MM:ss YYYY format\n");
+			  exit(1);
+			}
+
+		      if (bar[3] != ' ' || bar[6] != ' ' ||
+			  bar[15] != ' ' || bar[9] != ':' ||
+			  bar[12] != ':' || !isdigit(bar[5]) ||
+			  !isdigit(bar[7]) || !isdigit(bar[8]) ||
+			  !isdigit(bar[10]) || !isdigit(bar[11]) ||
+			  !isdigit(bar[13]) || !isdigit(bar[14]) ||
+			  !isdigit(bar[16]) || !isdigit(bar[17]) ||
+			  !isdigit(bar[18]) || !isdigit(bar[19]) ||
+			  !(bar[4] == ' ' || bar[4] == '0' ||
+			    bar[4] == '1' || bar[4] == '2' ||
+			    bar[4] == '3'))
+			{
+			  fprintf(stderr, "fatal: the date must be in the "
+				  "Jan DD HH:MM:ss YYYY format\n");
+			  exit(1);
+			}
+		      memset(config->begin_date, ' ', 4);
+		      memcpy(config->begin_date + 4, argv[i + 1], 21);
+		    }
+		  else
+		    {
+		      fprintf(stderr, "fatal: the +B parameter requires a "
+			      "path following it.\n");
+		      exit(1);
+		    }
+		  j = strlen(argv[++i]);
+		  continue;
+		case 'E':
+		  if (i < (argc - 1) && *argv[i + 1] != '-' &&
+		      *argv[i + 1] != '+')
+		    {
+		      char * bar = argv[i + 1];
+
+		      if (strlen(argv[i + 1]) != 20)
+			{
+			  fprintf(stderr, "fatal: the date must be in the "
+				  "Jan DD HH:MM:ss YYYY format\n");
+			  exit(1);
+			}
+
+		      if (bar[3] != ' ' || bar[6] != ' ' ||
+			  bar[15] != ' ' || bar[9] != ':' ||
+			  bar[12] != ':' || !isdigit(bar[5]) ||
+			  !isdigit(bar[7]) || !isdigit(bar[8]) ||
+			  !isdigit(bar[10]) || !isdigit(bar[11]) ||
+			  !isdigit(bar[13]) || !isdigit(bar[14]) ||
+			  !isdigit(bar[16]) || !isdigit(bar[17]) ||
+			  !isdigit(bar[18]) || !isdigit(bar[19]) ||
+			  !(bar[4] == ' ' || bar[4] == '0' ||
+			    bar[4] == '1' || bar[4] == '2' ||
+			    bar[4] == '3'))
+			{
+			  fprintf(stderr, "fatal: the date must be in the "
+				  "Jan DD HH:MM:ss YYYY format\n");
+			  exit(1);
+			}
+		      memset(config->end_date, ' ', 4);
+		      memcpy(config->end_date + 4, argv[i + 1], 21);
+		    }
+		  else
+		    {
+		      fprintf(stderr, "fatal: the +E parameter requires a "
+			      "path following it.\n");
+		      exit(1);
+		    }
+		  j = strlen(argv[++i]);
+		  continue;
 #if 0 /* This will have to wait for another version */
 		case 'T':
 		  if (i < (argc - 1) && *argv[i + 1] != '-' &&
@@ -2124,11 +2239,7 @@ parse_cmdline(int argc, char *argv[], config_t * config)
 		      }
 		    else
         	      {
-			if (!(config->only_dir = g_strdup(argv[i + 1])))
-			  {
-			    perror("parse_cmdline: g_strdup");
-			    exit(1);
-			  }
+			config->only_dir = g_strdup(argv[i + 1]);
 			config->only_dir_length = strlen(argv[i + 1]);
 		      }
 		  else
@@ -2170,11 +2281,7 @@ parse_cmdline(int argc, char *argv[], config_t * config)
       else
 	{
 	  /* it's a filename */
-	  if (!(logfile_name = g_strdup(argv[i])))
-	    {
-	      perror("parse_cmdline: g_strdup(argv[i])");
-	      exit(1);
-	    }
+	  logfile_name = g_strdup(argv[i]);
 
 	  if (!cmdline_logfile)
 	    {
@@ -2278,7 +2385,11 @@ xferstats_init(pointers_t *pointers)
 
   pointers->config->max_report_size = 30;
   pointers->config->depth = 4;
-  pointers->low_date[0] = pointers->high_date[0] = 0;
+  pointers->low_date[0] =
+    pointers->high_date[0] =
+    pointers->config->begin_date[0] =
+    pointers->config->end_date[0] = 0;
+
 #if 0
   /* This will have to wait for another version */
   pointers->config->only_tld = NULL;
@@ -2301,7 +2412,19 @@ xferstats_init(pointers_t *pointers)
     pointers->first_file_ptr =
     pointers->first_dir_ptr =
     pointers->first_size_ptr = NULL;
-  pointers->file_count = pointers->data = 0;
+  pointers->file_count =
+    pointers->data =
+    pointers->config->refresh =
+    pointers->config->chunk_input = 0;
+
+  pointers->file_ht =
+    pointers->tld_ht =
+    pointers->domain_ht =
+    pointers->host_ht =
+    pointers->dir_ht =
+    pointers->size_ht = NULL;
+
+  pointers->done = 0;
 
   /* allocate and clean out these arrays.  these arrays are generated in this
    * fashion instead of statically so that I can use libc's qsort().  I hope
@@ -2400,83 +2523,73 @@ main(int argc, char **argv)
     }
 
   /* load the xferlog into ftp_line structures */
-  switch(pointers->config->log_type)
-  {
-  case 0:
-    parse_wuftpd_logfile(pointers);
-    break;
-  case 1:
-    parse_ncftpd_logfile(pointers);
-    break;
-  case 2:
-    parse_apache_logfile(pointers);
-    break;
-  }
+  while (!pointers->done) {
+    switch(pointers->config->log_type) {
+    case 0:
+      if (!parse_wuftpd_logfile(pointers))
+	pointers->done = 1;
+      break;
+    case 1:
+      if (!parse_ncftpd_logfile(pointers))
+	pointers->done = 1;
+      break;
+    case 2:
+      if (!parse_apache_logfile(pointers))
+	pointers->done = 1;
+      break;
+    }
 
-  G_BLOW_CHUNKS();
+    G_BLOW_CHUNKS();
 
-  if (!pointers->first_ftp_line)
-    {
+    if (!pointers->first_ftp_line) {
       fprintf(stderr, "No data to process.\n");
       exit(0);
     }
 
-  generate_daily_data(pointers, pointers->first_ftp_line);
+    generate_daily_data(pointers, pointers->first_ftp_line);
+    if (pointers->config->dow_traffic)
+      generate_dow_data(pointers, pointers->first_ftp_line);
+    if (pointers->config->dom_traffic)
+      generate_dom_data(pointers, pointers->first_ftp_line);
+    if (pointers->config->monthly_traffic)
+      generate_monthly_data(pointers, pointers->first_ftp_line);
+    if (pointers->config->file_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_FILE);
+    if (pointers->config->dir_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_DIR);
+    if (pointers->config->host_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_HOST);
+    if (pointers->config->domain_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_DOMAIN);
+    if (pointers->config->tld_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_TLD);
+    if (pointers->config->hourly_traffic)
+      generate_hourly_data(pointers, pointers->first_ftp_line);
+    if (pointers->config->size_traffic)
+      generate_misc_data(pointers, pointers->first_ftp_line, T_SIZE);
+  }
+
   display_daily_totals(pointers);
   if (pointers->config->dow_traffic)
-    {
-      generate_dow_data(pointers, pointers->first_ftp_line);
-      display_dow_totals(pointers);
-    }
+    display_dow_totals(pointers);
   if (pointers->config->dom_traffic)
-    {
-      generate_dom_data(pointers, pointers->first_ftp_line);
-      display_dom_totals(pointers);
-    }
+    display_dom_totals(pointers);
   if (pointers->config->monthly_traffic)
-    {
-      generate_monthly_data(pointers, pointers->first_ftp_line);
-      display_monthly_totals(pointers);
-    }
-
+    display_monthly_totals(pointers);
   if (pointers->config->file_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_FILE);
-      display_misc_totals(pointers, T_FILE);
-    }
+    display_misc_totals(pointers, T_FILE, "file.html");
   if (pointers->config->dir_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_DIR);
-      display_misc_totals(pointers, T_DIR);
-    }
-
+    display_misc_totals(pointers, T_DIR, "dir.html");
   if (pointers->config->host_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_HOST);
-      display_misc_totals(pointers, T_HOST);
-    }
+    display_misc_totals(pointers, T_HOST, "host.html");
   if (pointers->config->domain_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_DOMAIN);
-      display_misc_totals(pointers, T_DOMAIN);
-    }
+    display_misc_totals(pointers, T_DOMAIN, "domain.html");
   if (pointers->config->tld_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_TLD);
-      display_misc_totals(pointers, T_TLD);
-    }
-
+    display_misc_totals(pointers, T_TLD, "tld.html");
   if (pointers->config->hourly_traffic)
-    {
-      generate_hourly_data(pointers, pointers->first_ftp_line);
-      display_hourly_totals(pointers);
-    }
-
+    display_hourly_totals(pointers);
   if (pointers->config->size_traffic)
-    {
-      generate_misc_data(pointers, pointers->first_ftp_line, T_SIZE);
-      display_misc_totals(pointers, T_SIZE);
-    }
+    display_misc_totals(pointers, T_SIZE, "size.html");
 
 #ifdef DEBUGS
   /* display memory information to stderr */
