@@ -2,7 +2,7 @@
 #define _XFERSTATS_H
 
 /*
- * Copyright 1997, 1998 Phil Schwan <pschwan@cmu.edu>
+ * Copyright 1997-1999 Phil Schwan <pschwan@cmu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,7 +53,7 @@
 #  define G_MEM_CHUNK_FREE g_mem_chunk_free
 #endif
 
-#define VERSION "2.12"
+#define VERSION "2.14"
 
 /* Also self-explanitory.  The default place to look for the xferstats config
  * information */
@@ -276,6 +276,15 @@ typedef struct _config_t
 
   /* Should we strictly check the log file to try to weed out corrupt lines? */
   char strict_check;
+
+  /* filter all traffic before this date */
+  char begin_date[25];
+  /* filter all traffic after this date */
+  char end_date[25];
+
+  int chunk_input;
+
+  unsigned int refresh;
 } config_t;
 
 typedef struct
@@ -315,6 +324,15 @@ typedef struct
   /* the first and last day that we have data for */
   char low_date[25];
   char high_date[25];
+
+  GHashTable * file_ht;
+  GHashTable * dir_ht;
+  GHashTable * host_ht;
+  GHashTable * tld_ht;
+  GHashTable * domain_ht;
+  GHashTable * size_ht;
+
+  char done;
 } pointers_t;
 
 /* prototypes */
@@ -323,15 +341,18 @@ int init_config(pointers_t * pointers);
 
 /* display.c */
 void display_daily_totals(pointers_t * pointers);
-void display_misc_totals(pointers_t * pointers, int type);
+void display_misc_totals(pointers_t * pointers, int type, char * htmlname);
 void display_hourly_totals(pointers_t * pointers);
 void display_monthly_totals(pointers_t * pointers);
 void display_dow_totals(pointers_t * pointers);
 void display_dom_totals(pointers_t * pointers);
 
 /* parselog.c */
-void parse_wuftpd_logfile(pointers_t * pointers);
-void parse_ncftpd_logfile(pointers_t * pointers);
-void parse_apache_logfile(pointers_t * pointers);
+int parse_wuftpd_logfile(pointers_t * pointers);
+int parse_ncftpd_logfile(pointers_t * pointers);
+int parse_apache_logfile(pointers_t * pointers);
+
+/* xferstats.c */
+inline int compare_dates(char date1[25], char date2[25]);
 
 #endif /* _XFERSTATS_H */
